import argparse
from . import tar_database  as tardb

def tardb_create():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i','--input', type = str, help = 'path to store the tardb')
    args = parser.parse_args()
    tardb.create_tardb(args.input)


def tardb_print_entries():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i','--input', type = str, help = 'path to store the tardb')
    args = parser.parse_args()
    print(tardb.list_entries(args.input))


def tardb_add_entries():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type = str, help = 'path to store the tardb')
    parser.add_argument('-el', '--entrylist', nargs='+', help = 'list of entries to add')
    args = parser.parse_args()
    tardb.add_entries(args.input, *args.entrylist)


def tardb_delete_entry():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i' '--input', type = str, help = 'path to store the tardb')
    parser.add_argument('-e', '--entry', type = str, help = 'entry to delete')
    args = parser.parse_args()
    tardb.add_entries(args.input, args.entry)


def tardb_print_entry():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type = str, help = 'path to store the tardb')
    parser.add_argument('-e', '--entry', type = str, help = 'entry to print (in full)')
    args = parser.parse_args()
    print(tardb.get_entry(args.input, args.entry))



from __future__ import annotations

import re
from pint import UnitRegistry

# ------------------------------------------------------------------------------
# Unit Conversion wrapper
# ------------------------------------------------------------------------------

ureg = UnitRegistry()
ureg.define("psia = psi")
ureg.define("psid = psi")
ureg.define("gpm = gal/min")
ureg.define("lbm = lb")


def convert(value: float, in_units: str, out_units: str):
    """
    Converts a unit from in_units to out_units
    Compatible Strings: https://github.com/hgrecco/pint/blob/master/pint/default_en.txt
    """
    if in_units == out_units:
        return value

    _u = ureg.Quantity(value, in_units)

    return _u.to(out_units).magnitude


def get_brack_units(string: str):
    """
    Returns the units inside the first []
    """
    return re.findall(r"\[(.*?)\]", string)[0]


def string_to_imperial(string: str):

    def replace(match):

        string = match.group(0)
        if string == "[watt/(m*degK)]":
            return string.replace("watt/(m*degK)", "Btu/(hr*ft*degF)")

        if "m" in string:
            string = re.sub(r"\bm\b", "in", string)

        if "kg" in string:
            string = string.replace("kg", "lbm")

        if "N" in string:
            string = string.replace("N", "lbf")

        if "J" in string:
            string = string.replace("J", "ft*lbf")

        if "degK" in string:
            string = string.replace("degK", "degF")

        if "Pa" in string:
            string = string.replace("Pa", "psia")

        if "watt" in string:
            string = string.replace("watt", "ft*lbf/s")

        return string

    # Replace the text within brackets based on different conditions
    return re.sub(r"\[.*?\]", replace, string)

def get_val_from_string(string: str, out_units: str):

    if '[-]' in string:
        return float(string.split('[-]')[0])

    # Hacky way to do this but its working?  Should fix at some poitn
    if ' [' in string:
        in_units = get_brack_units(string)
        number = float(eval(string.split(' [')[0]))
        return convert(number, in_units, out_units)
    else:
        return float(string)

def imperial_dictionary(dictionary):
    imperial = {}

    for key, value in dictionary.items():
        new_key = string_to_imperial(key)
        current_units = get_brack_units(key)

        new_units = get_brack_units(new_key)
        try:
            imperial[new_key] = convert(value, current_units, new_units)
        except:
            raise ValueError(f" CANT CONVERT: {value}, {current_units} to {new_units}")

    return imperial



from __future__ import annotations
import polars as pl
import plotly.graph_objects as go
from copy import deepcopy
import re

# Exposing it so anything that uses eval can convert
from .units import convert


def parse_keys(full_str: str, ):

    if ('{' and '}' not in full_str):
        return full_str

    keys = re.findall(r"\{(.*?)\}", full_str)
    new_str = deepcopy(full_str)
    for key in keys:
        new_str = new_str.replace('{' + key + '}', f'pl.col("{key}")')

    return 'df.with_columns([( + ' + new_str + ').alias("' + full_str + '")])'


def graph_scatter_by_key(
    df: pl.DataFrame,
    x: str,
    y: str,
    x_title=None,
    y_title='',
    title='',
    color=None,
    mode='lines',
    group_name: str = None,
    options: dict = None,
    fig: go.Figure = None,
    axis=1,
    theme='plotly_dark',
    alt_y_name=None,
):

    if options is None:
        options = {}

    if x_title is None:
        x_title = x

    if fig is None:
        fig = go.Figure()

    if alt_y_name is None:
        alt_y_name = y

    y_axis_info = dict(
        title=dict(text=y_title),
        anchor="free",
        overlaying="y",
        autoshift=True,
        side="left"
    )

    if color is not None:
        y_axis_info['color'] = color

    if y not in df:
        df_evaluated = eval(parse_keys(y))
    else:
        df_evaluated = df

    # TODO: Fix these copies?
    data = dict(
        x=df_evaluated[x].to_list(),
        y=df_evaluated[y].to_list(),
        name=alt_y_name,
        mode=mode,
        legendgroup=group_name,
        legendgrouptitle_text=group_name,
        **options
    )

    if mode == 'lines':
        data['line'] = dict(color=color)
    elif mode == 'markers':
        data['marker'] = dict(color=color)
    elif mode == 'lines+markers':
        data['line'] = dict(color=color)
        data['marker'] = dict(color=color)

    if axis == 1:
        fig.add_trace(go.Scatter(
            **data
        ))
        fig.update_layout(yaxis=dict(title=dict(text=y_title)))
    elif axis == 2:
        fig.add_trace(go.Scatter(
            yaxis='y2',
            **data
        ))
        fig.update_layout(
            yaxis2=y_axis_info,
        )
    elif axis == 3:
        fig.add_trace(go.Scatter(
            yaxis='y3',
            **data
        ))
        fig.update_layout(
            yaxis3=y_axis_info,
        )
    elif axis == 4:
        fig.add_trace(go.Scatter(
            yaxis='y4',
            **data
        ))
        fig.update_layout(
            yaxis4=y_axis_info,
        )

    fig.update_xaxes(title=x_title)
    fig.update_layout(
        title=title,
        template=theme,
        showlegend=True,
        hovermode='x unified',
    )

    return fig


def graph_scatter_by_keylist(
    df: pl.DataFrame,
    x: str,
    y_list: list[str],
    x_title=None,
    y_title='',
    title='',
    color=None,
    mode='lines',
    group_name: str = None,
    options: dict = None,
    fig: go.Figure = None,
    axis=1,
    theme='plotly_dark',
    alt_y_dict=None,
):
    if fig is None:
        fig = go.Figure()

    if alt_y_dict is None:
        alt_y_dict = {}

    for y in y_list:
        fig = graph_scatter_by_key(
            df=df,
            x=x,
            y=y,
            x_title=x_title,
            y_title=y_title,
            title=title,
            color=color,
            mode=mode,
            group_name=group_name,
            options=options,
            fig=fig,
            axis=axis,
            theme=theme,
            alt_y_name=alt_y_dict.get(y),
        )

    return fig


def graph_all(
    df: pl.DataFrame,
    x: str,
    x_title: str = None,
    y_title: str = '',
    title: str = '',
    color: str = None,
    mode: str = 'lines',
    group_name: str = None,
    options: dict = None,
    fig: go.Figure = None,
    axis: int = 1,
    theme: str = 'plotly_dark',
    alt_y_dict: dict = None,
):
    key_list = [key.name for key in df if key.name != x]

    return graph_scatter_by_keylist(
        df=df,
        x=x,
        y_list=key_list,
        x_title=x_title,
        y_title=y_title,
        title=title,
        color=color,
        mode=mode,
        group_name=group_name,
        options=options,
        fig=fig,
        axis=axis,
        theme=theme,
        alt_y_dict=alt_y_dict,
    )


def graphs_containing(
    df,
    x,
    contains_list,
    not_containing_list=None,
    mode: str = 'lines',
    theme: str = 'plotly_dark',
    options: dict = None
) -> list[go.Figure]:

    figs = []
    for contents in contains_list:
        key_list = [key.name for key in df if contents in key.name]

        if not_containing_list is not None:
            for not_key in not_containing_list:
                key_list = [key for key in key_list if not_key not in key]

        if len(key_list) > 0:
            figs.append(graph_scatter_by_keylist(
                df=df,
                x=x,
                y_list=key_list,
                mode=mode,
                options=options,
                theme=theme,
            ))

    return figs


def figs_to_html(*figs, prefix: str = None, export_path: str = None, include_plotlyjs: bool = False):

    if prefix is None:
        prefix = ''

    for fig in figs:
        prefix += fig.to_html(full_html=False,
                              include_plotlyjs=include_plotlyjs)

    if export_path is not None:
        with open(export_path, '+w') as f:
            f.write(prefix)

    return prefix



import plotly.graph_objects as go 
import polars as pl

def polars_to_html(df: pl.DataFrame) -> str:
    return df._repr_html_()

def fig_to_html(fig: go.Figure) -> str:
    return fig.to_html(full_html = False)

def collapsable(html: str, title: str) -> str:
    return f'<details><summary>{title}</summary>' + html + '</details>'